// @flow

import React, { Component } from 'react';

type Props = {
  className?: string,
  required: boolean,
  touched: boolean,
  error: void | [string],
  name: string,
  id?: string, // If not set, the name is used as the id
  placeholder?: string,
  type: string,
  onFocus?: (SyntheticEvent<HTMLInputElement>) => void,
  onBlur?: (SyntheticEvent<HTMLInputElement>) => void,
};

type LocalState = {|
  hasFocus: boolean,
|};

class FormElementInput extends Component<Props, LocalState> {
  static displayName = 'FormElementInput';

  state = {
    hasFocus: false,
  };

  onFocus = (evt: SyntheticEvent<HTMLInputElement>) => {
    this.setState({
      hasFocus: true,
    });

    const { onFocus } = this.props;
    if (onFocus) {
      onFocus(evt);
    }
  };

  onBlur = (evt: SyntheticEvent<HTMLInputElement>) => {
    this.setState({
      hasFocus: false,
    });

    const { onBlur } = this.props;
    if (onBlur) {
      onBlur(evt);
    }
  };

  render() {
    const {
      placeholder = undefined,
      required,
      touched,
      error,
      name,
      type,
      id = this.props.name,
      className,
      ...miscProps
    } = this.props;

    const classNames = [];
    if (error) {
      classNames.push('invalid');
    }

    if (!error && touched && !this.state.hasFocus) {
      classNames.push('valid');
    }

    const elmProps = {
      ...miscProps,
      className: `${className || ''} ${classNames.join(' ')}`,
      name,
      id,
      'aria-required': required,
      'aria-invalid': !!error,
      'aria-describedby': `${id}_description`,
      placeholder,
      type,
    };

    return <input {...elmProps} onFocus={this.onFocus} onBlur={this.onBlur} />;
  }
}

FormElementInput.displayName = 'FormElementInput';

export default FormElementInput;
