// @flow

import React, { Component } from 'react';

import type { Node as ReactNode } from 'react';
import styles from './styles.css';

type Props = {
  className?: string,
  required: boolean,
  checked: boolean,
  touched: boolean,
  error: void | [string],
  name: string,
  children: ReactNode, // value for label
  value: string,
  onFocus?: (SyntheticEvent<HTMLInputElement>) => void,
  onBlur?: (SyntheticEvent<HTMLInputElement>) => void,
};

type LocalState = {|
  hasFocus: boolean,
|};

class FormElementRadio extends Component<Props, LocalState> {
  static displayName = 'FormElementRadio';

  state = {
    hasFocus: false,
  };

  onFocus = (evt: SyntheticEvent<HTMLInputElement>) => {
    this.setState({
      hasFocus: true,
    });

    const { onFocus } = this.props;
    if (onFocus) {
      onFocus(evt);
    }
  };

  onBlur = (evt: SyntheticEvent<HTMLInputElement>) => {
    this.setState({
      hasFocus: false,
    });

    const { onBlur } = this.props;
    if (onBlur) {
      onBlur(evt);
    }
  };

  render() {
    const {
      required,
      touched,
      error,
      name,
      className,
      children,
      ...miscProps
    } = this.props;

    const classNames = [];
    if (error) {
      classNames.push('invalid');
    }

    if (!error && touched && !this.state.hasFocus) {
      classNames.push('valid');
    }

    const id = `${this.props.name}-${encodeURIComponent(this.props.value)}`;
    const describedBy = `${id}_description`;

    const elmProps = {
      ...miscProps,
      className: `${className || ''} ${classNames.join(' ')}`,
      name,
      id,
      'aria-required': required,
      'aria-invalid': !!error,
      'aria-describedby': describedBy,
    };

    return (
      <div className={styles.wrapper}>
        <input
          {...elmProps}
          onFocus={this.onFocus}
          onBlur={this.onBlur}
          type="radio"
          className={styles.radio}
        />
        <label htmlFor={id} id={describedBy} className={styles.label}>
          {children}
        </label>
      </div>
    );
  }
}

FormElementRadio.displayName = 'FormElementRadio';

export default FormElementRadio;
