// @flow

// import 'systema-spinner/styles.css';

import type { ChildrenArray, Element } from 'react';
import React, { Component } from 'react';

type Props = {
  className?: string,
  required: boolean,
  touched: boolean,
  error: void | [string],
  name: string,
  id?: string, // If not set, the name is used as the id
  children: ChildrenArray<Element<'option'> | Element<'optgroup'>>,
  onFocus?: (SyntheticEvent<HTMLSelectElement>) => void,
  onBlur?: (SyntheticEvent<HTMLSelectElement>) => void,
};

type LocalState = {|
  hasFocus: boolean,
|};

class FormElementSelect extends Component<Props, LocalState> {
  static displayName = 'FormElementSelect';
  state = {
    hasFocus: false,
  };

  onFocus = (evt: SyntheticEvent<HTMLSelectElement>) => {
    this.setState({
      hasFocus: true,
    });

    const { onFocus } = this.props;
    if (onFocus) {
      onFocus(evt);
    }
  };

  onBlur = (evt: SyntheticEvent<HTMLSelectElement>) => {
    this.setState({
      hasFocus: false,
    });

    const { onBlur } = this.props;
    if (onBlur) {
      onBlur(evt);
    }
  };

  render() {
    const {
      required,
      touched,
      error,
      name,
      id = this.props.name,
      className,
      ...miscProps
    } = this.props;

    const classNames = [];
    if (error) {
      classNames.push('invalid');
    }

    if (!error && touched && !this.state.hasFocus) {
      classNames.push('valid');
    }

    const elmProps = {
      ...miscProps,
      className: `${className || ''} ${classNames.join(' ')}`,
      name,
      id,
      'aria-required': required,
      'aria-invalid': !!error,
      'aria-describedby': `${id}_description`,
    };

    return <select {...elmProps} onFocus={this.onFocus} onBlur={this.onBlur} />;
  }
}

FormElementSelect.displayName = 'FormElementSelect';

export default FormElementSelect;
