// @flow

import { horizontal, inline, vertical } from './align';

import type { Align as AlignType } from './align';
import FormButtonBar from './button-bar';
import FormElementInput from './elements/input';
import FormElementRadio from './elements/radio';
import FormElementSelect from './elements/select';
import FormElementTextArea from './elements/textarea';
import FormGroup from './group';
import FormItem from './item';
import { isRequired } from './utils';
import { createForm as rcCreateForm } from 'rc-form';

type CreateForm = (
  option?: CreateFormOptions
) => <C: ComponentType<*>>(C) => ComponentType<FormProps & ElementProps<C>>;
const createForm: CreateForm = rcCreateForm;

export type Align = AlignType;

export type CreateFormOptions = {
  validatemessages?: { [string]: string | { [string]: string } },
  mapProps?: ({ [string]: any }) => { [string]: any },
  onFieldsChange?: (
    props: { [string]: any },
    changedValues: { [string]: any }
  ) => void,
  mapPropsToFields?: (props: { [string]: any }) => { [string]: any },
  withRef?: boolean,
};

export type ValidationRuleType =
  | 'string'
  | 'number'
  | 'boolean'
  | 'method'
  | 'regexp'
  | 'integer'
  | 'float'
  | 'array'
  | 'object'
  | 'enum'
  | 'date'
  | 'url'
  | 'hex'
  | 'email';

export type ValidationRuleRange = { min: number, max: number };

export type AsyncValidatorRule = {|
  type?: string,
  required?: boolean,
  pattern?: RegExp,
  range?: ValidationRuleRange,
  length?: number,
  enum?: [any],
  transform?: (value: string) => any,
  message?: ReactNode,
  validator?: (
    rule: AsyncValidatorRule,
    value: any,
    callback: (string) => any
  ) => void,
|};

export type ValidateLogic = {|
  trigger: string | [string], // default: [] (means it will only validate when validateFields is called)
  rules: AsyncValidatorRule,
|};

export type FormFieldOptions = {|
  valuePropName?: string, // default: 'value'
  getValueProps?: (value: any) => { [string]: any }, // default: (value: <C>) => { value: C }
  initialValue?: any, // default: undefined
  normalize?: (value: any, prev: any, allValues: { [string]: any }) => any, // default: (value) => value
  trigger?: string, // default: 'onChange'
  validateTrigger?: void | null | false | string | [string], // default: []
  rules?: [AsyncValidatorRule], // default undefined (no rules)
  validateFirst?: boolean, // default false
  fieldNameProp?: string, // default undefined
  fieldMetaProp?: string, // default undefined
  validate?: [ValidateLogic], // default undefined
  getValueFromEvent?: (SyntheticEvent<*>) => any, // default
|};

export type GetFieldProps = (
  string,
  FormFieldOptions
) => {
  ref: ElementRef<*>,
  onBlur?: (evt: SyntheticEvent<*>) => void,
  onChange?: (evt: SyntheticEvent<*>) => void,
  onFocus?: (evt: SyntheticEvent<*>) => void,
  value?: string,
};

type GetFieldError = string => void | [string];
type IsSubmitting = () => boolean;
type IsFieldTouched = string => boolean;
type ValidateFields = (error: void | {}, values: any) => void;
type Submit = (callback: () => void) => void;

export type FormProps = {
  form: {
    getFieldProps: GetFieldProps,
    getFieldError: GetFieldError,
    getFieldValue: (name: string) => any,
    isSubmitting: IsSubmitting,
    isFieldTouched: IsFieldTouched,
    validateFields: ValidateFields,
    submit: Submit,
  },
};

export {
  horizontal,
  vertical,
  inline,
  FormButtonBar,
  FormItem,
  FormGroup,
  FormGroup as default,
  FormElementInput,
  FormElementRadio,
  FormElementSelect,
  FormElementTextArea,
  isRequired,
  createForm,
};
