// @flow

import type { Align } from '../align';
import React from 'react';
import type { Node as ReactNode } from 'react';
import { horizontal } from '../align';
import styles from './styles.css';

type Props = {
  htmlFor: string,
  required: boolean,
  error: void | [string],
  label: ReactNode,
  children: ReactNode,
  helpText?: ReactNode,
  align: Align,
};

const FormItem = (props: Props) => {
  const { htmlFor, required, label, children, error, helpText, align } = props;

  const labelClassnames = [];
  if (required) {
    labelClassnames.push(styles.required);
  }

  if (error) {
    labelClassnames.push(styles.labelError);
  }

  return (
    <div className={styles[`${align}-item`]}>
      <div className={styles[`${align}-label`]}>
        <label htmlFor={htmlFor} className={labelClassnames.join(' ')}>
          {label}
        </label>
      </div>
      <div className={styles[`${align}-input`]}>
        {children}
        <div role="alert" id={`${htmlFor}_description`}>
          {error && <div className={styles[`${align}-error`]}>{error}</div>}
          {helpText && (
            <div className={styles[`${align}-help`]}>{helpText}</div>
          )}
        </div>
      </div>
    </div>
  );
};

FormItem.displayName = 'FormItem';

FormItem.defaultProps = {
  align: horizontal,
};

export default FormItem;
